-----------------------------------------------------------------------------------------------
-- Client Lua Script for WeaponSwapper
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- WeaponSwapper Module Definition
-----------------------------------------------------------------------------------------------
local WeaponSwapper = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local knVersion = "0.1"
	 
local glog
local GeminiLogging

local kstrDefaultLogLevel = "DEBUG"
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function WeaponSwapper:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function WeaponSwapper:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		"Gemini:Logging-1.2"
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- WeaponSwapper OnLoad
-----------------------------------------------------------------------------------------------
function WeaponSwapper:OnLoad()
	GeminiLogging = Apollo.GetPackage("Gemini:Logging-1.2").tPackage
    glog = GeminiLogging:GetLogger({
        level = GeminiLogging.WARN,
        pattern = "%d %n %c %l - %m",
        appender = "GeminiConsole"
    })
	self.log = glog
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("WeaponSwapper.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	
end

-----------------------------------------------------------------------------------------------
-- WeaponSwapper OnDocLoaded
-----------------------------------------------------------------------------------------------
function WeaponSwapper:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "WeaponSwapForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		Apollo.RegisterSlashCommand("ws", "OnSlashCmd", self)
		self.timer = ApolloTimer.Create(2.0, false, "OnTimer", self)
		
		Apollo.RegisterEventHandler("PlayerResurrected", "OnResurrect", self)
		Apollo.RegisterEventHandler("PlayerEquippedItemChanged", "OnEquipItemChange", self)
		
		if not self.tOffsets then 
			self.tOffsets = {-75,-100,75,35} 
		end		
		self.wndMain:SetAnchorOffsets(unpack(self.tOffsets))
		
		self.tBag = self.wndMain:FindChild("ToEquipBag")
		self.wndBagContainer = self.wndMain:FindChild("BagContainer")
		self.nTimesReequipped = 0
		self.tWeapon = nil
		self.tTempWeapon = nil
		self.bFirstLoad = true
		self.bJustDied = false
		self.bMovable = false
	end
end

-----------------------------------------------------------------------------------------------
-- EventHandler Functions
-----------------------------------------------------------------------------------------------

function WeaponSwapper:OnSlashCmd()
	self.bMovable = not self.bMovable
	self.wndMain:SetStyle("Moveable", self.bMovable)
	self.wndMain:Show(self.bMovable, true)
end

function WeaponSwapper:OnTimer()
	if self.bFirstLoad then
		self:InitWeapon()
		self.bFirstLoad = false
	end
end

function WeaponSwapper:OnResurrect()
	if self.tWeapon and self.tTempWeapon then
		if self:HasImpact(self.tWeapon) then
			self.bJustDied = true
			self:InitWindow()
		end
	else self:WeaponErrorMsg()
	end
end

function WeaponSwapper:OnEquipItemChange()
glog:debug("JustDied %s", tostring(self.bJustDied))
	if self.bJustDied and self:HasImpact(self.tWeapon) then
		self.nTimesReequipped = self.nTimesReequipped + 1
		glog:debug("TimesRe %d", self.nTimesReequipped)
		local tEquippedWeapon = self.tWeapon:GetEquippedItemForItemType()
		glog:debug(tEquippedWeapon:GetName())
		if self.nTimesReequipped == 1 and tEquippedWeapon:GetName() == self.tTempWeapon:GetName() then
			self:InitWindow()
		elseif self.nTimesReequipped == 2 and tEquippedWeapon:GetName() == self.tWeapon:GetName() then
			self.bJustDied = false			
			self:InitWindow()
			self.nTimesReequipped = 0
		end
	else 
		self:InitWeapon()
	end
end
---------------------------------------------------------------------------------------------------
-- Init Functions
---------------------------------------------------------------------------------------------------

function WeaponSwapper:InitWindow()
	if self.nTimesReequipped < 3 and self.bJustDied then
		self.tBag:SetSort(true)
		self.tBag:SetItemSortComparer(self.SortItemBySet)
		self.wndMain:Show(true, true)
	else
		self.wndMain:Show(false, true)
	end		
end

function WeaponSwapper:InitWeapon()
	local tPlayer = GameLib.GetPlayerUnit()
	if not tPlayer then return end
	local tEquipped = tPlayer:GetEquippedItems()
	local tBagContents = tPlayer:GetInventoryItems()		

	for idx,tItem in pairs(tEquipped) do
		if tItem:GetSlot() == 16 then --16 is "primary weapon"
			if (self.tWeapon and not self:IsSameItem(self.tWeapon, tItem)) or not self.tWeapon then
				self.tWeapon = tItem
			end
		end
	end
	
	for idx,tBagItem in pairs(tBagContents ) do
		if tBagItem then
		local tItem = tBagItem.itemInBag
			if ((tItem:CanEquip() and tItem:IsSoulbound()) and tItem:GetSlot() == 16) and 
				(not self.tTempWeapon and not self:IsSameItem(self.tWeapon, tItem)) then
				self.tTempWeapon = tItem
			end	
		end
	end

	if not self.tTempWeapon then
		self:WeaponErrorMsg()		
	end
	
end

---------------------------------------------------------------------------------------------------
-- Item Checker Functions
---------------------------------------------------------------------------------------------------

function WeaponSwapper:IsSameItem(tOrigItem, tNewItem)
	if tOrigItem == nil or tNewItem == nil then return false end
	
	if tOrigItem:GetName() ~= tNewItem:GetName() then return false end
		
	local tODetailedInfo = tOrigItem:GetDetailedInfo().tPrimary
	local tNDetailedInfo = tNewItem:GetDetailedInfo().tPrimary
	if tODetailedInfo.arBudgetBasedProperties == nil then
		return tODetailedInfo.arBudgetBasedProperties == nil
	elseif tNDetailedInfo.arBudgetBasedProperties == nil then
		return false
	end
	
	for i=1,15 do
		if tODetailedInfo.arBudgetBasedProperties[i] == nil then
			return tNDetailedInfo.arBudgetBasedProperties[i] == nil
		end
		if tNDetailedInfo.arBudgetBasedProperties[i]== nil then return false end
		
		if tODetailedInfo.arBudgetBasedProperties[i].nSortOrder ~= tNDetailedInfo.arBudgetBasedProperties[i].nSortOrder then
			return false
		end
		if tODetailedInfo.arBudgetBasedProperties[i].eProperty ~= tNDetailedInfo.arBudgetBasedProperties[i].eProperty then
			return false
		end
		if tODetailedInfo.arBudgetBasedProperties[i].nValue ~= tNDetailedInfo.arBudgetBasedProperties[i].nValue then
			return false
		end
	end
	return true
end

function WeaponSwapper:WeaponErrorMsg()
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "You need a second usable weapon for WeaponSwapper to work.")
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "Due to API problems, Addons can not directly equip/unequip items and so we need to have a 2nd weapon to switch in.")
end

function WeaponSwapper:HasImpact(tItem)
	if not tItem then return false end
	if tItem:GetEquipSpell() then
		local strSpecial = tItem:GetEquipSpell():GetName()
		if strSpecial == "Impact" then return true else return false end
	else 
		return false
	end
end

function WeaponSwapper.SortItemBySet(itemLeft, itemRight)
	local ws = Apollo.GetAddon("WeaponSwapper")
	if ws == nil then
		-- return 0
	end
	
	if itemLeft == itemRight then 
		return 0 
	end
	
	local leftInSet = ws:IsWeapon(itemLeft)
	local rightInSet = ws:IsWeapon(itemRight)

	if leftInSet and not rightInSet then
		return -1
	end
	
	if rightInSet and not leftInSet then
		return 1
	end
	
	if itemLeft == nil then
		return -1
	end
	
	if itemRight == nil then
		return 1
	end
	
  	local strLeftName = itemLeft:GetName()
  	local strRightName = itemRight:GetName()
 	if strLeftName < strRightName then
    	return -1
  	end
  	if strLeftName > strRightName then
    	return 1
  	end
  	
  	return 0
end

function WeaponSwapper:IsWeapon(tItem)
	if tItem:CanEquip() then
		if tItem:GetName() == self.tWeapon:GetName() or tItem:GetName() == self.tTempWeapon:GetName() then
			return true
		else
			return false
		end
	else
		return false
	end
end
---------------------------------------------------------------------------------------------------
-- Save/Load Functions
---------------------------------------------------------------------------------------------------


function WeaponSwapper:OnSave(eLevel)
	if eLevel~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	
	local tSavedData = {}
	tSavedData.tOffsets = self.tOffsets
	tSavedData.knVersion = knVersion
	return tSavedData
end

function WeaponSwapper:OnRestore(eLevel, tData)
self.tSavedData = tData
	if eLevel~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	if tData then
		if tData.knVersion == knVersion then 
			self.tOffsets = tData.tOffsets
		end
	end
end

---------------------------------------------------------------------------------------------------
-- WeaponSwapForm Functions
---------------------------------------------------------------------------------------------------

function WeaponSwapper:OnWindowMove( wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom )
	local X1, Y1, X2, Y2 = wndHandler:GetAnchorPoints()
	local x1, y1, x2, y2 = wndHandler:GetAnchorOffsets()
	self.tAnchors = {X1, Y1, X2, Y2}
	self.tOffsets = {x1, y1, x2, y2}
	glog:debug(self.tOffsets)
end

-----------------------------------------------------------------------------------------------
-- WeaponSwapper Instance
-----------------------------------------------------------------------------------------------
local WeaponSwapperInst = WeaponSwapper:new()
WeaponSwapperInst:Init()
